/******************************************************************************
Copyright 2005 Stephen Appling

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
******************************************************************************/
var WshShell = new ActiveXObject("WScript.Shell");
var regBase = "HKCU\\Software\\sappling\\ss2xml\\";


function makeIndent(indent)
{
   var indentString = "";
   for (var i=0; i<indent; i++)
   {
      indentString += "\t";
   }
   return indentString;
}

function escapeToXML(data)
{
   data = data.replace(/&/,"&amp;");
   data = data.replace(/</,"&lt;");
   return data;
}



function SS2XML()
{
   try
   {  this.user = WshShell.RegRead(regBase+"user");  }
   catch (e)
   {  this.user = "guest";  }

   try
   {  this.ssdb = WshShell.RegRead(regBase+"ssdb"); }
   catch (e)
   {  this.ssdb = "/srcsafe.ini";  }

   try
   {  this.location = WshShell.RegRead(regBase+"location"); }
   catch (e)
   {  this.location = "$/";  }


   this.pass = "";
   this.outpath = "ss.xml";
   this.stopAtLabel = "";
   this.ignoreLabel = "";
   this.onlyChanged = true;
   this.listener = null;
   this.projectList = new Array();
}

function SS2XML_open()
{
   try
   {
   this.ss = new ActiveXObject("SourceSafe");
   this.ss.Open(this.ssdb, this.user, this.pass);
   this.rootItem = this.ss.VSSItem(this.location);
   WshShell.RegWrite (regBase+"user", this.user, "REG_SZ");
   WshShell.RegWrite (regBase+"ssdb", this.ssdb, "REG_SZ");
   WshShell.RegWrite (regBase+"location", this.location, "REG_SZ");

   return this.rootItem;
   }
   catch (e)
   {
      throw e;
   }
}

function SS2XML_registerListener(func)
{
   this.listener = func;
}

function SS2XML_getCheckoutNames(item)
{
   var results = new Array();
   var e = new Enumerator(item.Checkouts);
   var itemArray = new Array();
   for (;!e.atEnd();e.moveNext())
   {
      itemArray.push(e.item());
   }
   for (var i=0; i<itemArray.length; i++)
   {
      var checkout = itemArray[i];
      itemArray[i] = null;
      results.push(checkout.UserName);
   }
   return results;
}

function SS2XML_getHistory(item, indent)
{
   var indentString = makeIndent(indent);
   var e = new Enumerator(item.Versions);
   var itemArray = new Array();
   for (;!e.atEnd();e.moveNext())
   {
      itemArray.push(e.item());
   }
   var results = "";
   for (var i=0; i<itemArray.length; i++)
   {
      var version = itemArray[i];
      itemArray[i] = null;
      if ((this.stopAtLabel.length >0) && (version.Label == this.stopAtLabel))
      {
         break;
      }
      if ((this.ignoreLabel.length == 0) || (version.Label != this.ignoreLabel))
      {
         results += indentString +'<version action="'+version.Action+'" date="'+version.Date+'" label="'+version.Label+'" username="'+version.Username+'" number="'+version.VersionNumber+'">'+
                    escapeToXML(version.Comment) + '</version>\n';
      }
   }
   return results;
}

function SS2XML_getFile(item, indent)
{
   if (this.listener)
   {
      this.listener("File   : "+item.Spec);
   }
   var history = this.getHistory(item, indent+1); 
   var results = "";
   if (history.length > 0 || !this.onlyChanged)
   {
      var indentString = makeIndent(indent);
      var openLine = indentString + '<file name="'+item.Name+'" fullpath="'+item.Spec+'" '+
                     'workingfile="'+item.LocalSpec+'" deleted="'+item.Deleted+'" binary="'+item.Binary+
                     '" currentversion="'+item.VersionNumber+'"';
      if (item.IsCheckedOut)
      {
         openLine += ' checkouts="'+this.getCheckoutNames(item)+'" ';
      }
      openLine += ">\n";                                           
      results += openLine + history + indentString + '</file>\n'
   }
   return results;
}

function SS2XML_getItem(item, indent)
{
   var results = "";
   if (item.Type == 0) // if it's a project
   {
      if (this.listener)
      {
         this.listener("Project: "+item.Spec);
      }

      var indentString = makeIndent(indent);
      var e = new Enumerator(item.Items);
      var itemArray = new Array();
      for (;!e.atEnd();e.moveNext())
      {
         itemArray.push(e.item());
      }
      e = null;
      for (var i=0; i<itemArray.length; i++)
      {
         results += this.getItem(itemArray[i], indent +1);
         itemArray[i] = null;
      }
      if (results.length > 0)
      {
         results = indentString +'<project name="'+item.Name+'" fullpath="'+item.Spec+'" workingfolder="'+item.LocalSpec+'">\n' +
                    results + indentString +'</project>\n';
      }
   }
   else
   {
      results = this.getFile(item, indent);
   }
   return results;
}

function SS2XML_exportMetaData(outputStream)
{
   outputStream.WriteLine("<?xml version=\"1.0\"?>");
   outputStream.WriteLine("<!DOCTYPE project SYSTEM \"ss2xml.dtd\">");
   outputStream.Write(converter.getItem(converter.open(), 0));
}

function SS2XML_getLabels(item)
{
   var e = new Enumerator(item.Versions(0x00001000 + 0x40000000));
   var itemArray = new Array();
   var results = new Array();

   for (;!e.atEnd();e.moveNext())
   {
      itemArray.push(e.item());
   }
   for (var i=0; i<itemArray.length; i++)
   {
      var version = itemArray[i];
      itemArray[i] = null;
      if (version.Label.length > 0)
      {
         results.push(escapeToXML(version.Label));
      }
   }
   return results;
}

function SS2XML_getUsers()
{
   var e = new Enumerator(this.ss.Users);
   var itemArray = new Array();
   for (;!e.atEnd();e.moveNext())
   {
      itemArray.push(e.item());
   }
   return itemArray;
}

function SS2XML_addProjectListItem(item)
{
   if (item.Type == 0) // if it's a project
   {
      if (this.listener)
      {
         this.listener(item.Spec);
      }
      this.projectList.push(item);
      var e = new Enumerator(item.Items);
      for (;!e.atEnd();e.moveNext())
      {
         this.addProjectListItem(e.item());
      }
   }
}


// private, not added to prototype
function SS2XML_formatRightsAsXML(rights, parentRights, location, indent)
{
   var result="";
   if (!(rights & 16))
   {
      result = makeIndent(indent) +"<rights project=\""+location+"\">\n";
      var innerIndent = makeIndent(indent+1);
      if (rights & 1)
      {
         result += innerIndent + "<read/>\n";
      }
      if (rights & 2)
      {
         result += innerIndent + "<check-in-out/>\n";
      }
      if (rights & 4)
      {
         result += innerIndent + "<add-rename/>\n";
      }
      if (rights & 8)
      {
         result += innerIndent + "<destroy/>\n";
      }
      result += makeIndent(indent) +"</rights>\n";
   }
   return result;
}


function SS2XML_formatUserAsXML(user, indent)
{
   var result = makeIndent(indent);
   result += '<user name="'+user.Name+'">\n';
   for (var i=0; i<this.projectList.length; i++)
   {
      var location = this.projectList[i];
      var parentRights;
      if (location.Parent.Spec != location.Spec)
      {
         parentRights = user.ProjectRights(location.Parent.Spec);
      }
      else
      {
         parentRights = -1;
      }

      result += SS2XML_formatRightsAsXML(user.ProjectRights(location.Spec), parentRights,  location.Spec, indent+1);
   }
   result += makeIndent(indent) + "</user>\n";
   return result;
}

function SS2XML_exportUsers(outputStream, onlyRoot )
{
   if (this.ss.ProjectRightsEnabled && !onlyRoot)
   {
      if (this.listener)
      {
         this.listener("Building list of all projects:");
      }
      this.addProjectListItem(this.rootItem);
   }
   else
   {
      this.projectList.push(this.rootItem);
   }

   if (this.listener)
   {
      this.listener("\nGetting Users:");
   }
   var users = this.getUsers();
   outputStream.WriteLine("<?xml version=\"1.0\"?>");
   outputStream.WriteLine("<!DOCTYPE users SYSTEM \"ssusers.dtd\">");
   outputStream.WriteLine("<users>");
   for (var i=0; i<users.length; i++)
   {
      if (this.listener)
      {
         this.listener(users[i].Name);
      }
      outputStream.Write(this.formatUserAsXML(users[i], 1));
   }
   outputStream.WriteLine("</users>");
}


SS2XML.prototype.open = SS2XML_open;
SS2XML.prototype.registerListener = SS2XML_registerListener;
SS2XML.prototype.getCheckoutNames = SS2XML_getCheckoutNames;
SS2XML.prototype.getHistory = SS2XML_getHistory;
SS2XML.prototype.getFile = SS2XML_getFile;
SS2XML.prototype.getItem = SS2XML_getItem;
SS2XML.prototype.getLabels = SS2XML_getLabels;
SS2XML.prototype.addProjectListItem = SS2XML_addProjectListItem;
SS2XML.prototype.setDatabase = function(val) { this.ssdb = val; };
SS2XML.prototype.getDatabase = function() { return this.ssdb; };
SS2XML.prototype.setUser = function(val) { this.user = val; };
SS2XML.prototype.getUser = function(val) { return this.user; };
SS2XML.prototype.setPassword = function(val) { this.pass = val; };
SS2XML.prototype.setLocation = function(val) { this.location = val; };
SS2XML.prototype.getLocation = function(val) { return this.location; };
SS2XML.prototype.setStopAtLabel = function(val) { this.stopAtLabel = val; };
SS2XML.prototype.setIgnoreLabel = function(val) { this.ignoreLabel = val; };
SS2XML.prototype.setOnlyChanged = function(val) { this.onlyChanged = val; };
SS2XML.prototype.getRootItem = function() { return this.rootItem; };
SS2XML.prototype.getUsers = SS2XML_getUsers;
SS2XML.prototype.exportUsers = SS2XML_exportUsers;
SS2XML.prototype.exportMetaData = SS2XML_exportMetaData;
SS2XML.prototype.formatUserAsXML = SS2XML_formatUserAsXML;

/*
function SSAdmin()
{
}
*/


